import { Context, Recogniser } from "./encoding";
export interface Match {
    confidence: number;
    name: string;
    lang?: string;
}
declare const _default: (ctx: Context, rec: Recogniser, confidence: number) => Match;
export default _default;
